program NCRibbon;

{
This is my way to thank all of you at CIS for all the
programming-stuff I learned and ideas I got from you!

If you have any questions or suggestions on this ribbon-program,
just mail me at CIS 100034, 2645

If you plan to alter the ribbon: share it with others!

========================================================================

This demonstration program shows you how you can set up a ribbon bar and
how to use it within your application.

The principle is simple:
- define a variable of type PRibbon and create it
- create a list of all available buttons [AddButton and AddSeparator]
- show it {ShowRibbon]

Be sure to add the call to ReSize in your wm_Size handler.

=========================================================================

You can setup your own buttons using the Resource Workshop.

"Limitations" so far:
- use BWCC
- do not use button-ID's already in use by BWCC
- do not use 999 as button-ID: it's in used by the static-control in
  the ribbon editor window

}

{$R NCRIBBON.RES }

uses
  BWCC,
  ODialogs,
  OWindows,
  WinCrt,
  WinProcs,
  WinTypes,
  Win31,
  Ribbon;

const
  ApplName         = 'NC Ribbon, version 1.1';

  cm_FilePrint     = 101;
  cm_RibbonShow    = 201;
  cm_RibbonHide    = 202;
  cm_HelpIndex     = 998;
  cm_HelpAbout     = 999;

type
  TMyApplication   = object(TApplication)
    procedure   InitMainWindow; virtual;
  end; { TMyApplication }

  PMyWindow        = ^TMyWindow;
  TMyWindow        = object( TWindow )
    Ribbon         : PRibbon;  { to be created in SetupWindow }
    constructor Init( aParent: PWindowsObject; aTitle: PChar );
    function    GetClassName: PChar; virtual;
    procedure   SetupWindow; virtual;
    procedure   WMSize( var Msg: TMessage );
      virtual wm_First + wm_Size;

    procedure   CMFileNew( var Msg: TMessage );
      virtual cm_First + cm_FileNew;
    procedure   CMFileOpen( var Msg: TMessage );
      virtual cm_First + cm_FileOpen;
    procedure   CMFileSave( var Msg: TMessage );
      virtual cm_First + cm_FileSave;

    procedure   CMRibbonShow( var Msg: TMessage );
      virtual cm_First + cm_RibbonShow;
    procedure   CMRibbonHide( var Msg: TMessage );
      virtual cm_First + cm_RibbonHide;

    procedure   CMHelpAbout( var Msg: TMessage );
      virtual cm_First + cm_HelpAbout;
  end; { TMyWindow }

var
  MyApp            : TMyApplication;

{=========================================================================}
{== Implementation of TMyWindow                                         ==}
{=========================================================================}

constructor TMyWindow.Init( aParent: PWindowsObject; aTitle: PChar );

begin
  inherited Init( aParent, aTitle );
  with Attr do
    begin
      Menu := LoadMenu( HInstance, 'MainMenu' );
      X := 0;
      Y := 0;
      W := 640;
      H := 480;
    end; { with }

  { Here is where we define our ribbon. By specifying 'rbn_EnableEdit' you
    will be able to edit the ribbon-contents. If you do not want your
    users to edit the ribbon, just use 'rbn_DisableEdit' }
  Ribbon := New( PRibbon, Init( @Self, rbn_EnableEdit ));
end; { TMyWindow.Init }

{=========================================================================}

function TMyWindow.GetClassName: PChar;

begin
  GetClassName := 'RibbonTestProgram';
end; { TMyWindow.GetClassName }

{=========================================================================}

procedure TMyWindow.SetupWindow;

begin
  inherited SetupWindow;
  with Ribbon^ do
    begin
      { At this place we define all available buttons. We do this by
        calling AddButton with the following parameters:
        - BtnID :  This is the ID under which you define your bitmaps
                   in the Resource Workshop, for example:
                     use 101 for your images 1101 (btn-up) and
                     3101 (btn-down).
        - CmdID :  This is the command-ID you specified for your menu-
                   item. It will be send to the mainwindow when you
                   press a ribbon-button.
        - Descr :  Just a short description that will be shown in
                   the 'Edit_Ribbon' dialog.
        - Flag  :  TRUE if the button has to be active at startup,
                   otherwise false.
      }
      AddButton( 208, cm_HelpIndex, 'Display help index', TRUE );
      { The simplest way to add a separator...}
      AddSeparator;
      AddButton( 201, cm_MDIFileNew, 'Create a new file', TRUE );
      AddButton( 202, cm_MDIFileOpen, 'Open an existing file', TRUE );
      AddButton( 203, cm_FileSave, 'Save active document', TRUE );
      AddButton( 204, cm_FilePrint, 'Print active document', TRUE );
      AddSeparator;
      AddButton( 205, cm_EditCut, 'Cut selection to clipboard', TRUE );
      AddButton( 206, cm_EditCopy, 'Copy selection to clipboard', TRUE );
      AddButton( 207, cm_EditPaste, 'Paste clipboard to document', TRUE );
      { ...and just show them... }
      ShowButtons;
    end; { with }
end; { TMyWindow.SetupWindow }

{=========================================================================}

procedure TMyWindow.WMSize(var Msg: TMessage);

var
  Rect             : TRect;

begin
  getUpdateRect( Hwindow, Rect, False);
  sendmessage( HWindow, wm_SetRedraw, 0, 0);
  inherited WMSize(Msg);

  { don't forget to add this statement to your wm_Size handler!}
  if Ribbon <> nil then
    Ribbon^.ReSize( Msg );

  sendmessage( HWindow, wm_SetRedraw, 1 ,0);
  RedrawWindow( HWindow, @Rect,0,
	 rdw_InValidate or rdw_UpdateNow or rdw_AllChildren );
end; { TMyWindow.WMSize }

{=========================================================================}

procedure TMyWindow.CMFileNew( var Msg: TMessage );

begin
  BWCCMessageBox( HWindow, 'FileNew', 'You choose...', mb_OK );
end; { TMyWindow.CMFileNew }

{=========================================================================}

procedure TMyWindow.CMFileOpen( var Msg: TMessage );

begin
  BWCCMessageBox( HWindow, 'FileOpen', 'You choose...', mb_OK );
end; { TMyWindow.CMFileOpen }

{=========================================================================}

procedure TMyWindow.CMFileSave( var Msg: TMessage );

begin
  BWCCMessageBox( HWindow, 'FileSave', 'You choose...', mb_OK );
end; { TMyWindow.CMFileSave }

{=========================================================================}

procedure TMyWindow.CMRibbonShow( var Msg: TMessage );

begin
  { You may show or hide the bar at runtime }
  if Ribbon <> nil then
    Ribbon^.Visible;
end; { TMyWindow.CMRibbonShow }

{=========================================================================}

procedure TMyWindow.CMRibbonHide( var Msg: TMessage );

begin
  if Ribbon <> nil then
    Ribbon^.Hide;
end; { TMyWindow.CMRibbonHide}

{=========================================================================}

procedure TMyWindow.CMHelpAbout( var Msg: TMessage );

var
  Dlg              : PDialog;

begin
  Dlg := New( PDialog, Init( @Self, 'About' ));
  Application^.ExecDialog( Dlg );
end; { TMyWindow.CMHelpAbout }

{=========================================================================}
{== Implementation of TMyApplication                                    ==}
{=========================================================================}

procedure TMyApplication.InitMainWindow;

begin
  MainWindow := New( PMyWindow, Init( nil, ApplName ));
end; { TMyApplication.InitMainWindow }

{=========================================================================}

begin
  MyApp.Init( ApplName );
  MyApp.Run;
  MyApp.Done;
end. { main }